<?php

namespace AppBuilder\Di\Service\Auth;

use AppBuilder\Di\App\Http\HttpClientInterface;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;
use WP_Error;
use WP_User;

/**
 * Class ForgotPassword
 *
 * @link       https://appcheap.io
 * @since      5.0.0
 * @author     ngocdt
 */
class ForgotPassword {
	use \AppBuilder\Traits\Permission;

	/**
	 * Http client
	 *
	 * @var HttpClientInterface $http_client Http client.
	 */
	private HttpClientInterface $http_client;

	/**
	 * ForgotPassword constructor.
	 *
	 * @param HttpClientInterface $http_client Http client.
	 */
	public function __construct( HttpClientInterface $http_client ) {
		$this->http_client = $http_client;
	}

	/**
	 * Register rest route.
	 */
	public function register_rest_route() {
		// Step 1: Forgot password with action send OTP to email.
		register_rest_route(
			'app-builder/v1',
			'/forgot-password',
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'forgot_password' ),
				'permission_callback' => array( $this, 'public_permissions_callback' ),
				'args'                => $this->schema( null ),
			)
		);
		register_rest_route(
			'app-builder/v1',
			'/auth/forgot-password',
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'forgot_password' ),
				'permission_callback' => array( $this, 'public_permissions_callback' ),
				'args'                => $this->schema( null ),
			)
		);

		// Step 2: Verify OTP and return user token.
		register_rest_route(
			'app-builder/v1',
			'/verify-otp-forgot-password',
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'verify_otp_forgot_password' ),
				'permission_callback' => array( $this, 'public_permissions_callback' ),
				'args'                => $this->schema( 'verify' ),
			)
		);
		register_rest_route(
			'app-builder/v1',
			'/auth/verify-otp-forgot-password',
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'verify_otp_forgot_password' ),
				'permission_callback' => array( $this, 'public_permissions_callback' ),
				'args'                => $this->schema( 'verify' ),
			)
		);

		// Step 3: Update new password.
		register_rest_route(
			'app-builder/v1',
			'/update-password',
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'update_password' ),
				'permission_callback' => array( $this, 'public_permissions_callback' ),
				'args'                => $this->schema( 'update' ),
			)
		);
		register_rest_route(
			'app-builder/v1',
			'/auth/update-password',
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'update_password' ),
				'permission_callback' => array( $this, 'public_permissions_callback' ),
				'args'                => $this->schema( 'update' ),
			)
		);
	}

	/**
	 * Forgot password.
	 *
	 * @param WP_REST_Request $request Request.
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function forgot_password( WP_REST_Request $request ): WP_REST_Response|WP_Error {
		$user_login = $request->get_param( 'user_login' );

		$user_data = $this->verify( $user_login );

		if ( is_wp_error( $user_data ) ) {
			return $user_data;
		}

		// Redefining user_login ensures we return the right case in the email.
		$user_login = $user_data->user_login;
		$user_email = $user_data->user_email;
		$key        = get_password_reset_key( $user_data );

		if ( is_wp_error( $key ) ) {
			return new WP_Error(
				'invalid_key',
				$key->get_error_message(),
				array(
					'status' => 403,
				)
			);
		}

		if ( is_multisite() ) {
			$site_name = get_network()->site_name;
		} else {
			/*
			 * The blogname option is escaped with esc_html on the way into the database
			 * in sanitize_option we want to reverse this for the plain text arena of emails.
			 */
			$site_name = wp_specialchars_decode( get_option( 'blogname' ), ENT_QUOTES );
		}

		// Generate OTP 6 digits.
		$key = wp_rand( 100000, 999999 );

		// Hi [User Name].
		$message = __( 'Hi', 'mobile-builder' ) . ' ' . $user_login . ',' . "\r\n\r\n";

		// We heard you're having trouble remembering your password for [App Name]. No worries, it happens to the best of us!
		$message .= __( 'We heard you\'re having trouble remembering your password for', 'mobile-builder' ) . ' ' . $site_name . '. ' . __( 'No worries, it happens to the best of us!', 'mobile-builder' ) . "\r\n\r\n";

		// Your OTP is: [6-digit OTP].
		$message .= __( 'Your OTP is:', 'mobile-builder' ) . ' ' . $key . "\r\n\r\n";

		// Please note: This OTP is confidential and should not be shared with anyone.
		$message .= __( 'Please note: This OTP is confidential and should not be shared with anyone.', 'mobile-builder' ) . "\r\n\r\n";

		// If you didn't request a password reset, please disregard this email. However, we recommend updating your password regularly to keep your account secure.
		$message .= __( 'If you didn\'t request a password reset, please disregard this email. However, we recommend updating your password regularly to keep your account secure.', 'mobile-builder' ) . "\r\n\r\n";

		// Subject: Access Your [App Name] Account again - One-Time Password Inside.
		$title = __( 'Access Your', 'mobile-builder' ) . ' ' . $site_name . ' ' . __( 'Account again - One-Time Password Inside', 'mobile-builder' );

		// Send email.
		if ( $message && ! wp_mail( $user_email, wp_specialchars_decode( $title ), $message ) ) {
			return new WP_Error(
				'send_email',
				__( 'Possible reason: your host may have disabled the mail() function.', 'mobile-builder' ),
				array(
					'status' => 403,
				)
			);
		}

		// Update OTP to user meta expire after 30 minutes.
		set_transient( 'app_builder_forgot_password_' . $user_data->ID, $key, 1800 );

		// Response.
		return rest_ensure_response( array( 'message' => __( 'OTP sent to your email.', 'mobile-builder' ) ) );
	}

	/**
	 * Verify OTP for forgot password.
	 *
	 * @param WP_REST_Request $request request.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function verify_otp_forgot_password( $request ): WP_Error|WP_REST_Response {
		$user_login = $request->get_param( 'user_login' );
		$otp        = $request->get_param( 'otp' );

		$user_data = $this->verify( $user_login );

		if ( is_wp_error( $user_data ) ) {
			return $user_data;
		}

		// Redefining user_login ensures we return the right case in the email.
		$user_login = $user_data->user_login;

		// Get OTP from user meta.
		$otp_user = get_transient( 'app_builder_forgot_password_' . $user_data->ID );

		// Check OTP.
		if ( $otp !== $otp_user ) {
			return new WP_Error(
				'invalid_otp',
				__( 'Invalid OTP', 'mobile-builder' ),
				array(
					'status' => 403,
				)
			);
		}

		$data = array(
			'user' => array(
				'id' => $user_data->ID,
			),
			'otp'  => $otp,
		);

		$token = app_builder()->get( 'token' )->sign_token( $user_data->ID, $data, 1800 );

		// Response.
		return rest_ensure_response(
			array(
				'token' => $token,
			)
		);
	}

	/**
	 * Update password for forgot password.
	 *
	 * @param WP_REST_Request $request request.
	 *
	 * @return WP_Error|WP_HTTP_Response|WP_REST_Response
	 */
	public function update_password( $request ) {
		$token        = $request->get_param( 'token' );
		$new_password = $request->get_param( 'new_password' );

		// Verify token.
		$data = app_builder()->get( 'token' )->verify_token( $token );

		if ( is_wp_error( $data ) ) {
			return $data;
		}

		$user_id = $data->data->user_id;
		$otp     = $data->data->otp;

		// Get OTP from user meta.
		$otp_user = get_transient( 'app_builder_forgot_password_' . $user_id );

		// Check OTP.
		if ( $otp !== $otp_user ) {
			return new WP_Error(
				'invalid_otp',
				__( 'Expired or Password changed', 'mobile-builder' ),
				array(
					'status' => 403,
				)
			);
		} else {
			// Delete OTP.
			delete_transient( 'app_builder_forgot_password_' . $user_id );
		}

		// Update new password.
		wp_set_password( $new_password, $user_id );

		// Response.
		return rest_ensure_response(
			array(
				'message' => __( 'Password updated', 'mobile-builder' ),
			)
		);
	}

	/**
	 * Verify user login.
	 *
	 * @param string $user_login user login.
	 *
	 * @return WP_Error|WP_User
	 */
	private function verify( $user_login ): WP_Error|WP_User {
		$errors = new WP_Error();
		if ( empty( $user_login ) || ! is_string( $user_login ) ) {
			$errors->add(
				'empty_username',
				__( 'Enter a username or email address.', 'mobile-builder' ),
				array(
					'status' => 404,
				)
			);
		} elseif ( strpos( $user_login, '@' ) ) {
			$user_data = get_user_by( 'email', trim( wp_unslash( $user_login ) ) );
			if ( empty( $user_data ) ) {
				$errors->add(
					'invalid_email',
					__( 'There is no account with that username or email address.', 'mobile-builder' ),
					array(
						'status' => 404,
					)
				);
			}
		} else {
			$login     = trim( $user_login );
			$user_data = get_user_by( 'login', $login );
		}

		if ( $errors->has_errors() ) {
			return $errors;
		}

		if ( ! $user_data ) {
			$errors->add(
				'invalidcombo',
				__( 'There is no account with that username or email address.', 'mobile-builder' ),
				array(
					'status' => 404,
				)
			);

			return $errors;
		}
		return $user_data;
	}

	/**
	 * Schema.
	 *
	 * @param string $type type of schema.
	 *
	 * @return array
	 */
	public function schema( string $type = null ): array {

		if ( 'update' === $type ) {
			return array(
				'token'        => array(
					'type'        => 'string',
					'required'    => true,
					'description' => 'Token',
				),
				'new_password' => array(
					'type'        => 'string',
					'required'    => true,
					'description' => 'New password',
				),
			);
		}

		if ( 'verify' === $type ) {
			return array(
				'user_login' => array(
					'type'        => 'string',
					'required'    => true,
					'description' => 'User login or email address',
				),
				'otp'        => array(
					'type'        => 'string',
					'required'    => true,
					'description' => 'OTP code',
				),
			);
		}

		return array(
			'user_login' => array(
				'type'        => 'string',
				'required'    => true,
				'description' => 'Username or email address',
			),
		);
	}
}
